from typing import Any, Dict, Optional

from graphql import graphql
from graphql.type import GraphQLSchema
from graphql.pyutils import is_awaitable

from .core import TypedGraphqlMiddlewareManager


async def await_awaitables(v: Any) -> Any:
    if is_awaitable(v):
        return await await_awaitables(await v)
    elif isinstance(v, dict):
        return {k: await await_awaitables(v) for k, v in v.items()}
    elif isinstance(v, list):
        return [await await_awaitables(v) for v in v]
    else:
        return v


async def execute_async(
    schema: GraphQLSchema,
    query: str,
    root: Any = None,
    context_value: Optional[Dict[str, Any]] = None,
):
    result = await graphql(
        schema,
        "{myUser {value {value}}}",
        root,
        context_value=context_value,
        middleware=TypedGraphqlMiddlewareManager(),
    )
    result.data = await await_awaitables(result.data)
    return result



from typing_inspect import is_typevar


def get_arg_for_typevar(typevar, generic):
    """Return the type of the TypeVar of this Generic"""

    assert is_typevar(typevar)

    # Search bases for the generic
    for base in generic.__orig_bases__:
        if base.__origin__.__orig_bases__[0].__args__[0] == typevar:
            return base.__args__[0]

    return None



from .core import (
    ReturnTypeMissing,
    TypedGraphqlMiddlewareManager,
    graphql_input_type,
    graphql_type,
    resolver,
    resolverclass,
    staticresolver,
)
from .execute import execute_async

__all__ = [
    "ReturnTypeMissing",
    "TypedGraphqlMiddlewareManager",
    "execute_async",
    "graphql_input_type",
    "graphql_type",
    "resolver",
    "resolverclass",
    "staticresolver",
]
